///////////////////////////////////////////////////////////////////////
// implementation of the CIDataMbx class
// last updated: 28oct98
// Copyright(C) 1998 Micronet Co. All Rights Reserved.

#include <string.h>
#include "rtpp.h"


///////////////////////////////////////////////////////////////////////
// constructor

CIDataMbx::			CIDataMbx(void)

	: CIObject(CITYPE_DATAMBX)
{
}


///////////////////////////////////////////////////////////////////////
// destructor

CIDataMbx::			~CIDataMbx(void)
{
	// detach the element
	// (code copied here to reach own Delete function)
	if (InternalDetach())
		Delete();
}


///////////////////////////////////////////////////////////////////////
// Create
// returns: zero or error code

DWORD CIDataMbx::	Create(
	char *				pszName,
	BOOL				bPrioQ)
{
	RTHANDLE		h;

	// detach from RT object
	Detach();

	// create the data mailbox
	h				= ::CreateRtMailbox(DATA_MAILBOX |
						(bPrioQ ? PRIORITY_QUEUING : FIFO_QUEUING));
	if (h == BAD_RTHANDLE)
		return (m_status = ::GetLastRtError());

	// attach the RT mailbox object (as owner)
	m_status		= E_OK;
	InternalAttach(pszName, h, TRUE);

	return m_status;
}


///////////////////////////////////////////////////////////////////////
// Send
// returns: zero or error code

DWORD CIDataMbx::	Send(
	PVOID				pv,
	WORD				wLen)
{
	if ((wLen < 1) || (wLen > 128))
		m_status	= E_PARAM;

	else if (m_hRTObject == NULL_RTHANDLE)
		m_status	= E_EXIST;

	else if (::SendRtData(m_hRTObject, pv, wLen))
		m_status	= E_OK;

	else
		m_status	= ::GetLastRtError();

	return m_status;
}


///////////////////////////////////////////////////////////////////////
// Receive
// returns: number of bytes received, or zero in case of error

WORD CIDataMbx::	Receive(
	PVOID				pv,
	WORD				wMax,
	DWORD				dwTimeout)
{
	WORD			wLen;

	wLen			= 0;
	if (wMax < 1)
		m_status	= E_PARAM;

	else if (m_hRTObject == NULL_RTHANDLE)
		m_status	= E_EXIST;

	else if (wMax >= 128)
	{
		// the supplied buffer is big enough: do direct receive
		wLen	= ::ReceiveRtData(m_hRTObject, pv, dwTimeout);
		if (wLen == 0)
			m_status= ::GetLastRtError();
		else
			m_status= E_OK;
	}
	else
	{
		// the buffer may be too small: use temporary buffer
		BYTE		byBuf[128];

		wLen		= ::ReceiveRtData(m_hRTObject, byBuf, dwTimeout);
		if (wLen == 0)
			m_status= ::GetLastRtError();

		else if (wLen > wMax)
			m_status= E_PARAM;

		else
		{
			memcpy(pv, byBuf, wLen);
			m_status= E_OK;
		}
	}
	return wLen;
}


///////////////////////////////////////////////////////////////////////
// Delete (protected)

void CIDataMbx::	Delete(void)
{
	// get rid of the RT data mailbox object
	if (m_hRTObject != NULL_RTHANDLE)
		if (::DeleteRtMailbox(m_hRTObject))
			m_status= E_OK;
		else
			m_status= ::GetLastRtError();
	else
		m_status	= E_PARAM;

	// do base class deletion
	CIObject::Delete();
}
