///////////////////////////////////////////////////////////////////////
// implementation of the CIIntHandler class
// last updated: 23Oct07
// Copyright(C) 2007 Micronet Co. All Rights Reserved.


#include "rtpp.h"

///////////////////////////////////////////////////////////////////////
// static class members

INTHANDLER CIIntHandler::
					m_pObjects[16]=
					{
						IRQ0_LEVEL, (CIIntHandler*)NULL,
						IRQ1_LEVEL, (CIIntHandler*)NULL,
						IRQ2_LEVEL, (CIIntHandler*)NULL,
						IRQ3_LEVEL, (CIIntHandler*)NULL,
						IRQ4_LEVEL, (CIIntHandler*)NULL,
						IRQ5_LEVEL, (CIIntHandler*)NULL,
						IRQ6_LEVEL, (CIIntHandler*)NULL,
						IRQ7_LEVEL, (CIIntHandler*)NULL,
						IRQ8_LEVEL, (CIIntHandler*)NULL,
						IRQ9_LEVEL, (CIIntHandler*)NULL,
						IRQ10_LEVEL, (CIIntHandler*)NULL,
						IRQ11_LEVEL, (CIIntHandler*)NULL,
						IRQ12_LEVEL, (CIIntHandler*)NULL,
						IRQ13_LEVEL, (CIIntHandler*)NULL,
						IRQ14_LEVEL, (CIIntHandler*)NULL,
						IRQ15_LEVEL, (CIIntHandler*)NULL
					};


///////////////////////////////////////////////////////////////////////
// constructor

CIIntHandler::		CIIntHandler(void)

	: CIObject(CITYPE_INTHANDLER)
{
	m_byMaxInt		= 0;
}


///////////////////////////////////////////////////////////////////////
// destructor

CIIntHandler::		~CIIntHandler(void)
{
	// detach the element
	// (code copied here to reach own Delete function)
	if (InternalDetach())
	{
		Delete();
		m_hRTObject	= NULL_RTHANDLE;
	}
}


///////////////////////////////////////////////////////////////////////
// Create
// returns: zero or error code

DWORD CIIntHandler::Create(
	WORD				wLevel,
	BOOL				bShared)
{
	int				i;

	Detach();

	m_wHardLevel	=
	m_wLevel		= wLevel;
	m_cName[0]		= 0;
	m_bShared		= bShared;

	SetRtProcessMaxPriority(NULL_RTHANDLE, 0);

	// attempt to set up interrupt handling
	if (!::SetRtInterruptHandler(m_wLevel, m_byMaxInt, InterruptHandler))
		Fail("Cannot set interrupt handler for level %x\n", wLevel);

	// change handle so it looks like a real object
	m_status		= E_OK;
	m_hRTObject		= BAD_RTHANDLE;
	m_bOwner		= TRUE;

	// put pointer to this object in the table
	// (no interrupt handler should use the table in the meantime)
	::knStopRtScheduler();
	for (i=0; i<16; i++)
		if (m_pObjects[i].wLevel == m_wHardLevel)
		{
			m_pObjects[i].pHandler	= this;
			break;
		}
	::knStartRtScheduler();

	return 0;
}


///////////////////////////////////////////////////////////////////////
// Delete (protected)

void CIIntHandler::	Delete(void)
{
	int				i;

	if (m_byMaxInt)
		m_status	= E_OK;
	else if (m_hRTObject != NULL_RTHANDLE)
		if (::ResetRtInterruptHandler(m_wLevel))
			m_status= E_OK;
		else
			m_status= ::GetLastRtError();
	else
		m_status	= E_PARAM;

	// clear pointer to this object in the table
	// (no interrupt handler should use the table in the meantime)
	::knStopRtScheduler();
	for (i=0; i<16; i++)
		if (m_pObjects[i].wLevel == m_wHardLevel)
		{
			m_pObjects[i].pHandler	= NULL;
			break;
		}
	::knStartRtScheduler();

	CIObject::Delete();
}


///////////////////////////////////////////////////////////////////////
// Enable
// returns: zero or error code

DWORD CIIntHandler::	Enable(
	BOOL				bState)
{
	BOOL			bResult;

	if (bState)
		bResult		= ::EnableRtInterrupt(m_wLevel);
	else
		bResult		= ::DisableRtInterrupt(m_wLevel);
	if (bResult)
		m_status	= ::GetLastRtError();
	else
		m_status	= E_OK;
	return m_status;
}


///////////////////////////////////////////////////////////////////////
// OnInterrupt
// This should contain the code to check the device, get status, etc. 
// Should return TRUE if the interrupt was for this handler, or
// FALSE if not. The FALSE return is most applicable to shared 
// handlers. 
BOOL CIIntHandler::	OnInterrupt(void)
{
	if (m_byMaxInt)
		return m_pThread->OnInterrupt();
	return TRUE;
}


///////////////////////////////////////////////////////////////////////
// Interrupthandler (static, generic non-shared interrupt handler)
// this handler assumes a local variable,
// and a call to OnInterrupt for every interrupt
// when overruling this handler, please read all rules on writing handlers

__INTERRUPT void CIIntHandler::
					InterruptHandler(void)
{
	WORD			wLevel;
	BOOL			found;
	int				i;
	INTHANDLER*		pObject;
	CIIntHandler*	pHandler;

	__INTERRUPT_PROLOG();

	// set up contxt for handler
	wLevel			= ::GetRtInterruptLevel();
	::EnterRtInterrupt(wLevel);

	// allow the OnInterrupt function to use low level functions
	::knStopRtScheduler();

	// find the object and call the OnInterrupt function
	found			= FALSE;
	for (i=0, pObject = m_pObjects; i < 16; i++, pObject++)
		if (pObject->wLevel == wLevel)
		{
			pHandler	= pObject->pHandler;
			if (pHandler)
			{
				if (pHandler->OnInterrupt())
					if (pHandler->m_byMaxInt)
					{
						SignalRtInterruptThread(wLevel);
						found	= TRUE;
					}
			}
			break;
		}

	// complete the interrupt
	if (!found)
		SignalEndOfRtInterrupt((BYTE)wLevel);

	// now allow thread switches to occur
	::knStartRtScheduler();

	__INTERRUPT_RETURN();
}
