///////////////////////////////////////////////////////////////////////
// implementation of the CIIntThread class
// last updated: 23Oct07
// Copyright(C) 2007 Micronet Co. All Rights Reserved.


#include "rtpp.h"


///////////////////////////////////////////////////////////////////////
// constructor

CIIntThread::		CIIntThread(void)

	: CIThread()
{
}


///////////////////////////////////////////////////////////////////////
// destructor

CIIntThread::		~CIIntThread(void)
{
	// detach the element
	// (code copied here to reach own Delete function)
	if (InternalDetach())
	{
		Delete();
		m_hRTObject	= NULL_RTHANDLE;
	}
}


///////////////////////////////////////////////////////////////////////
// Create
// returns: zero or error code

DWORD CIIntThread::	Create(
	char *				pszName,
	WORD				wLevel,
	BYTE				byMaxInt,
	DWORD				dwMilliseconds,
	DWORD				dwStack,
	BOOL				bShared)
{
	m_handler.m_pThread		= this;
	m_handler.m_byMaxInt	= byMaxInt;

	m_wLevel		= wLevel;
	m_dwMilliseconds= dwMilliseconds;

	// the remainder is the same as for CIThread
	return CIThread::Create(pszName, 0, dwStack);
}


///////////////////////////////////////////////////////////////////////
// Delete (protected)

void CIIntThread::	Delete(void)
{
	RTHANDLE		hTemp;


	m_handler.Delete();

	// since we may not come back from ResetRtInterruptHandler
	// make sure that the object is correct
	hTemp			= m_hRTObject;
	m_hRTObject		= NULL_RTHANDLE;
	if (hTemp != NULL_RTHANDLE)
		if (::ResetRtInterruptHandler(m_wLevel))
			m_status= E_OK;
		else {
			m_status= ::GetLastRtError();
			DeleteRtThread(hTemp);
		}
	else
		m_status	= E_PARAM;

	CIObject::Delete();
}


///////////////////////////////////////////////////////////////////////
// InitInstance
// returns: TRUE if successful, FALSE if app should terminate

BOOL CIIntThread::	InitInstance(void)
{
	if (m_handler.Create(m_wLevel, m_bShared))
		Fail("Cannot set interrupt handler for level %x\n", m_wLevel);

	m_wLevel		= m_handler.m_wLevel;
	m_status		= E_OK;
	return TRUE;
}


///////////////////////////////////////////////////////////////////////
// DoForever
// returns: TRUE if loop should continue, FALSE if thread should terminate

BOOL CIIntThread::	DoForever(void)
{
	// try to receive a signal
	if (!::WaitForRtInterrupt(m_wLevel, m_dwMilliseconds))
	{
		// error: if it was a timeout, do normal processing
		m_status	= ::GetLastRtError();
		if (m_status == E_TIME)
			return OnSignal(FALSE);
		else
			return FALSE;
	}
	else
		return OnSignal(TRUE);
}


///////////////////////////////////////////////////////////////////////
// OnInterrupt
// This should contain the code to check the device, get status, etc. 
// Should return TRUE if the interrupt was for this handler, or
// FALSE if not. The FALSE return is most applicable to shared 
// handlers. 
BOOL CIIntThread::	OnInterrupt(void)
{
	return TRUE;
}


///////////////////////////////////////////////////////////////////////
BOOL CIIntThread::	OnSignal(
	BOOL				bSignal)
{
	return TRUE;
}


///////////////////////////////////////////////////////////////////////
// Enable
// returns: zero or error code

DWORD CIIntThread::	Enable(
	BOOL				bState)
{
	return m_handler.Enable(bState);
}
