///////////////////////////////////////////////////////////////////////
// implementation of the CILLMbx class
// last updated: 28oct98
// Copyright(C) 1998 Micronet Co. All Rights Reserved.

#include <stdlib.h>
#include "rtpp.h"


///////////////////////////////////////////////////////////////////////
// constructor

CILLMbx::			CILLMbx(void)

	: CIObject(CITYPE_LLMBX)
{
}


///////////////////////////////////////////////////////////////////////
// destructor

CILLMbx::			~CILLMbx(void)
{
	// detach the element
	// (code copied here to reach own Delete function)
	if (InternalDetach())
		Delete();
}


///////////////////////////////////////////////////////////////////////
// Create

DWORD CILLMbx::		Create(
	DWORD				dwMsgSize,
	DWORD				dwQueueLength,
	BOOL				bPrioQ,
	BOOL				bPrioSlot)
{
	// detach from RT object
	Detach();

	// allocate space for the mailbox area
	m_dwMsgSize		= dwMsgSize;
	m_pArea			= (char*)malloc(KN_MAILBOX_SIZE +
						(dwMsgSize + KN_MAILBOX_MSG_OVERHEAD) *
						dwQueueLength);
	if (m_pArea == NULL)
		return (m_status = E_MEM);	

	// create the low level mailbox
	m_hLLObject		= ::knCreateRtMailbox(m_pArea,
						dwMsgSize,
						dwQueueLength,
						(bPrioQ ? KN_PRIORITY_QUEUING : KN_FIFO_QUEUING) |
						(bPrioSlot ? KN_RESERVE_PRIORITY_DATA : KN_DONT_RESERVE_PRIORITY_DATA));

	// now set the RT handle
	m_hRTObject		= BAD_RTHANDLE;
	m_bOwner		= TRUE;
	m_cName[0]		= 0;

	return (m_status = E_OK);
}


///////////////////////////////////////////////////////////////////////
// Send
// returns: zero or error code

DWORD CILLMbx::		Send(
	PVOID				pv,
	DWORD				dwLen)
{
	if (m_hRTObject == NULL_RTHANDLE)
		m_status	= E_EXIST;

	else if ((dwLen < 1) || (dwLen > m_dwMsgSize))
		m_status	= E_PARAM;

	else if (::knSendRtData(m_hLLObject, pv, dwLen))
		m_status	= E_OK;

	else
		m_status	= ::GetLastRtError();

	return m_status;
}


///////////////////////////////////////////////////////////////////////
// SendPriority
// returns: zero or error code

DWORD CILLMbx::		SendPriority(
	PVOID				pv,
	DWORD				dwLen)
{
	if (m_hRTObject == NULL_RTHANDLE)
		m_status	= E_EXIST;

	else if ((dwLen < 1) || (dwLen > m_dwMsgSize))
		m_status	= E_PARAM;

	else if (::knSendRtPriorityData(m_hLLObject, pv, dwLen))
		m_status	= E_OK;

	else
		m_status	= ::GetLastRtError();

	return m_status;
}


///////////////////////////////////////////////////////////////////////
// Receive
// returns: number of bytes received, or zero in case of error

DWORD CILLMbx::		Receive(
	PVOID				pv,
	DWORD				dwMax,
	DWORD				dwUsecs)
{
	DWORD			dwLen;

	dwLen			= 0;
	if (dwMax < 1)
		m_status	= E_PARAM;

	else if (m_hRTObject == NULL_RTHANDLE)
		m_status	= E_EXIST;

	else
	{
		if (!::knWaitForRtData(m_hLLObject,
				pv,
				&dwLen,
				CIApp::GetApp()->UsecsToKticks(dwUsecs)))
			m_status= ::GetLastRtError();
		else
			m_status= E_OK;
	}
	return dwLen;
}


///////////////////////////////////////////////////////////////////////
// Delete (protected)

void CILLMbx::		Delete(void)
{
	// get rid of the low level mailbox object
	if (m_hRTObject != NULL_RTHANDLE)
	{
		::knDeleteRtMailbox(m_hLLObject);
		free(m_pArea);
		m_pArea		= (char*)NULL;
		m_status= E_OK;
	}
	else
		m_status	= E_PARAM;

	// do base class deletion
	CIObject::Delete();
}
