///////////////////////////////////////////////////////////////////////
// implementation of the CILLPollThread and CILLPollThread2 classes
// last updated: 28oct98
// Copyright(C) 1998 Micronet Co. All Rights Reserved.


#include "rtpp.h"


///////////////////////////////////////////////////////////////////////
// constructor

CILLPollThread::	CILLPollThread(void)

	: CIThread()
{
	m_hAlarm		= (KNHANDLE)0;
	m_bChanged		= FALSE;
}


///////////////////////////////////////////////////////////////////////
// destructor

CILLPollThread::	~CILLPollThread()
{
	// forgetting to delete an alarm is disastrous!
	if (m_hAlarm != (KNHANDLE)0)
		knDeleteRtAlarmEvent(m_hAlarm);
}


///////////////////////////////////////////////////////////////////////
// Create
// returns: zero or error code

DWORD CILLPollThread::
					Create(
	char *				pszName,
	DWORD				dwUsecs,
	BYTE				byPrio,
	DWORD				dwStack,
	BOOL				bGo)
{
	Detach();

	m_dwTicks		= 	CIApp::GetApp()->UsecsToKticks(dwUsecs);

	// delete the alarm if it already exists
	if (m_hAlarm != (KNHANDLE)0)
		knDeleteRtAlarmEvent(m_hAlarm);

	// create the alarm
	m_hAlarm		= ::knCreateRtAlarmEvent(m_dwAlarm,
						KN_REPEATER,
						m_dwTicks);

	// create the thread
	CIThread::Create(pszName, byPrio, dwStack, bGo);

	return m_status;
}


///////////////////////////////////////////////////////////////////////
// SetPoll

void CILLPollThread::SetPoll(
	DWORD				dwUsecs)
{
	m_dwTicks		= 	CIApp::GetApp()->UsecsToKticks(dwUsecs);
	
	if (m_hAlarm != (KNHANDLE)0)
		m_bChanged	= TRUE;
}


///////////////////////////////////////////////////////////////////////
// DoForever
// returns: TRUE if loop should continue, FALSE if thread should terminate

BOOL CILLPollThread::DoForever(void)
{
	if (!::knWaitForRtAlarmEvent(m_hAlarm, KN_WAIT_FOREVER))
		Fail("Cannot wait for alarm");

	if (m_bChanged)
	{
		m_bChanged	= FALSE;

		// alarm is active: stop and start it again
		knDeleteRtAlarmEvent(m_hAlarm);

		m_hAlarm	= ::knCreateRtAlarmEvent(m_dwAlarm,
						KN_REPEATER,
						m_dwTicks);
	}
	return OnPoll();
}


///////////////////////////////////////////////////////////////////////

BOOL CILLPollThread::
					OnPoll(void)
{
	return TRUE;
}


///////////////////////////////////////////////////////////////////////
// constructor

CILLPollThread2::	CILLPollThread2(void)

	: CIThread()
{
	m_hAlarm		= (RTHANDLE)0;
	m_bChanged		= FALSE;
}


///////////////////////////////////////////////////////////////////////
// destructor

CILLPollThread2::	~CILLPollThread2()
{
	if (m_hAlarm != (RTHANDLE)0)
		DeleteRtAlarm(m_hAlarm);
}


///////////////////////////////////////////////////////////////////////
// Create
// returns: zero or error code

DWORD CILLPollThread2::
					Create(
	char *				pszName,
	DWORD				dwUsecs,
	BYTE				byPrio,
	DWORD				dwStack,
	BOOL				bGo)
{
	Detach();

	m_dwUsecs		= 	dwUsecs;

	// delete the alarm if it already exists
	if (m_hAlarm != (RTHANDLE)0)
		DeleteRtAlarm(m_hAlarm);

	// create the alarm
	m_hAlarm		= ::CreateRtAlarm(KN_REPEATER, m_dwUsecs);

	// create the thread
	CIThread::Create(pszName, byPrio, dwStack, bGo);

	return m_status;
}


///////////////////////////////////////////////////////////////////////
// SetPoll

void CILLPollThread2::SetPoll(
	DWORD				dwUsecs)
{
	m_dwUsecs		= dwUsecs;
	
	if (m_hAlarm != (RTHANDLE)0)
		m_bChanged	= TRUE;
}


///////////////////////////////////////////////////////////////////////
// DoForever
// returns: TRUE if loop should continue, FALSE if thread should terminate

BOOL CILLPollThread2::DoForever(void)
{
	if (!::WaitForRtAlarm(m_hAlarm, KN_WAIT_FOREVER))
		Fail("Cannot wait for alarm");

	if (m_bChanged)
	{
		m_bChanged	= FALSE;

		// alarm is active: stop and start it again
		DeleteRtAlarm(m_hAlarm);

		m_hAlarm	= ::CreateRtAlarm(KN_REPEATER, m_dwUsecs);
	}
	return OnPoll();
}


///////////////////////////////////////////////////////////////////////

BOOL CILLPollThread2::
					OnPoll(void)
{
	return TRUE;
}
