///////////////////////////////////////////////////////////////////////
// implementation of the CIMemory class
// last updated: 28oct98
// Copyright(C) 1998 Micronet Co. All Rights Reserved.

#include "rtpp.h"


///////////////////////////////////////////////////////////////////////
// constructor

CIMemory::			CIMemory(void)

	: CIObject(CITYPE_MEMORY)
{
	m_bAllocated	= FALSE;
}


///////////////////////////////////////////////////////////////////////
// destructor

CIMemory::			~CIMemory(void)
{
	// detach the RT object
	// (code copied here to reach own Delete function)
	if (InternalDetach())
		Delete();
}


///////////////////////////////////////////////////////////////////////
// Create
// returns: zero or error code

DWORD CIMemory::	Create(
	char *				pszName,
	DWORD				dwLen,
	PVOID				pAddr)
{
	RTHANDLE		h;

	// detach from RT object
	Detach();

	m_dwLen			= dwLen;

	// allocate the memory, if no address provided
	if (pAddr == NULL)
	{
		m_p			= ::AllocateRtMemory(dwLen);
		if (m_p == NULL)
			return (m_status = ::GetLastRtError());
		m_bAllocated= TRUE;
	}
	else
	{
		m_p			= pAddr;
		m_bAllocated= FALSE;
	}

	h				= ::CreateRtMemoryHandle(m_p, m_dwLen);
	if (h == BAD_RTHANDLE)
		return (m_status = ::GetLastRtError());

	// attach the memory (as owner)
	m_status		= E_OK;
	InternalAttach(pszName, h, TRUE);

	return m_status;
}


///////////////////////////////////////////////////////////////////////
// assignment

CIMemory & CIMemory::
					operator = (
	CIMemory &			src)
{
	if (this != &src)	// beware of self-assignment
	{
		// do basic class assignment
		CIObject::operator=((CIObject&)src);

		// copy class specific members
		m_p			= src.m_p;
		m_dwLen		= src.m_dwLen;
		m_bAllocated= FALSE;
	}
	return *this;
}


///////////////////////////////////////////////////////////////////////
// GetPointer
// returns: pointer or NULL

PVOID CIMemory::	GetPointer(void)
{
	if (m_hRTObject == NULL_RTHANDLE)
	{
		m_status	= E_EXIST;
		return NULL;
	}
	
	m_status		= E_OK;
	return m_p;
}


///////////////////////////////////////////////////////////////////////
// GetSize
// returns: size in bytes or 0

DWORD CIMemory::	GetSize(void)
{
	if (m_hRTObject == NULL_RTHANDLE)
	{
		m_status	= E_EXIST;
		return 0;
	}

	m_status		= E_OK;
	return m_dwLen;
}


///////////////////////////////////////////////////////////////////////
// Delete (protected)

void CIMemory::		Delete(void)
{
	if (m_hRTObject != NULL_RTHANDLE)
	{
		if (::DeleteRtMemoryHandle(m_hRTObject))
			if (!m_bAllocated || ::FreeRtMemory(m_p))
			{
				m_status= E_OK;
				return;
			}
		m_status	= ::GetLastRtError();
	}
	else
		m_status	= E_PARAM;

	CIObject::Delete();
}


///////////////////////////////////////////////////////////////////////
// remove INtime element (protected)

void CIMemory::		Remove(void)
{
	m_status		= E_OK;

	if (m_hRTObject != NULL_RTHANDLE)
		if (/*m_bAllocated &&*/ !::FreeRtMemory(m_p))
			m_status= ::GetLastRtError();
}


///////////////////////////////////////////////////////////////////////
// Lookup (no name, overruled)
// returns: zero or error code

DWORD CIMemory::	Lookup (
	char *				pszName,
	DWORD				dwTimeout)
{
	CIObject::Lookup(pszName, dwTimeout);
	if (m_status == E_OK)
	{
		// find settings (mapping a second time is a no-op)
		Map();
	}

	return m_status;
}


///////////////////////////////////////////////////////////////////////
// Attach (overruled)
// returns: zero or error code

DWORD CIMemory::	Attach(
	RTHANDLE			h)
{
	CIObject::Attach(h);
	if (m_status == E_OK)
	{
		// find settings (mapping a second time is a no-op)
		Map();
	}

	return m_status;
}


///////////////////////////////////////////////////////////////////////
// Map (protected)
// returns: zero or error code

DWORD CIMemory::	Map (void)
{
	// find settings (mapping a second time is a no-op)
	m_p				= ::MapRtSharedMemory(m_hRTObject);
	if (m_p == NULL)
	{
		m_hRTObject	= NULL_RTHANDLE;
		return (m_status= ::GetLastRtError());
	}

	m_dwLen			= ::GetRtSize(m_hRTObject);
	if (m_dwLen == 0)
	{
		m_status	= ::GetLastRtError();
		if (m_bAllocated)
			FreeRtMemory(m_p);
		m_hRTObject	= NULL_RTHANDLE;
		return m_status;
	}
	return (m_status = E_OK);
}
