///////////////////////////////////////////////////////////////////////
// implementation of the CIObjectMbx class
// last updated: 28oct98
// Copyright(C) 1998 Micronet Co. All Rights Reserved.

#include "rtpp.h"


///////////////////////////////////////////////////////////////////////
// constructor

CIObjectMbx::		CIObjectMbx(void)

	: CIObject(CITYPE_OBJECTMBX)
{
}


///////////////////////////////////////////////////////////////////////
// destructor

CIObjectMbx::		~CIObjectMbx(void)
{
	// detach the RT object
	// (code copied here to reach own Delete function)
	if (InternalDetach())
		Delete();
}


///////////////////////////////////////////////////////////////////////
// Create
// returns: zero or error code

DWORD CIObjectMbx::	Create(
	char *				pszName,
	BOOL				bPrioQ)
{
	RTHANDLE		h;

	// detach from RT object
	Detach();

	// create the object mailbox
	h				= ::CreateRtMailbox(OBJECT_MAILBOX |
						(bPrioQ ? PRIORITY_QUEUING : FIFO_QUEUING));
	if (h == BAD_RTHANDLE)
		return (m_status = ::GetLastRtError());

	// attach the RT mailbox object (as owner)
	m_status		= E_OK;
	InternalAttach(pszName, h, TRUE);

	return m_status;
}


///////////////////////////////////////////////////////////////////////
// Send
// returns: zero or error code

DWORD CIObjectMbx::	Send(
	CIObject *			pObject,
	CIObject *			pReply)
{
	RTHANDLE		hObject, hReply;

	m_status		= E_EXIST;

	// is the object mailbox attached?
	if (m_hRTObject == NULL_RTHANDLE)
		return m_status;

	// is the object OK?
	if (pObject == NULL)
		return m_status;
	hObject			= pObject->GetHandle();
	if (hObject == NULL_RTHANDLE)
		return m_status;

	if (pReply)
	{
		// sanity checks for the reply
		hReply		= pReply->GetHandle();
		if (hReply == NULL_RTHANDLE)
			return m_status;

		switch(pReply->GetType())
		{
		case CITYPE_DATAMBX:
		case CITYPE_OBJECTMBX:
		case CITYPE_SEMAPHORE:
			break;
		default:
			return (m_status = E_PARAM);// reply has bad type
		}
	}
	else
		hReply		= NULL_RTHANDLE;

	if (::SendRtHandle(m_hRTObject, hObject, hReply))
		m_status	= E_OK;
	else
		m_status	= ::GetLastRtError();

	return m_status;
}


///////////////////////////////////////////////////////////////////////
// Receive (with reply parameter)
// returns: object pointer or NULL

CIObject * CIObjectMbx::
					Receive(
	CIObject **			pReply,
	DWORD				dwTimeout)
{
	RTHANDLE		hObject, hReply;
	CIObject *		pObject;
	CIObject *		pTempReply;

	if (m_hRTObject == NULL_RTHANDLE)
		m_status	= E_EXIST;

	else
	{
		hObject		= ::ReceiveRtHandle(m_hRTObject, dwTimeout,
							&hReply);
		if (hObject == BAD_RTHANDLE)
			m_status= ::GetLastRtError();
		else
		{
			m_status= E_OK;
			pObject	= (CIObject*)NULL;
			switch (InternalGetType(hObject))
			{
			case CITYPE_DATAMBX:
				pObject	= new CIDataMbx();
				break;

			case CITYPE_OBJECTMBX:
				pObject	= new CIObjectMbx();
				break;

			case CITYPE_SEMAPHORE:
				pObject	= new CISemaphore();
				break;

			case CITYPE_REGION:
				pObject	= new CIRegion();
				break;

			case CITYPE_MEMORY:
				pObject	= new CIMemory();
				break;

			case CITYPE_THREAD:
				pObject	= new CIThread();
				break;
			}

			if (pObject)
			{
				pObject->Attach(hObject);

				// receive was OK; check reply
				if (pReply)
					if (hReply != NULL_RTHANDLE)
					{
						pTempReply	= new CIObject(InternalGetType(hReply));
						if (pTempReply)
						{
							pTempReply->Attach(hReply);
							*pReply		= pTempReply;
						}
						else
							m_status= E_MEM; // cannot create reply CIObject
					}
					else
						*pReply		= (CIObject*)NULL;

				else if (hReply != NULL_RTHANDLE)
					m_status= E_PARAM;
			}
			else
				m_status	= E_MEM; // cannot create a new CIObject
		}
	}

	return (m_status == E_OK) ? pObject : (CIObject*)NULL;
}


///////////////////////////////////////////////////////////////////////
// Receive (without reply parameter)
// returns: object pointer or NULL

CIObject * CIObjectMbx::
					Receive(
	DWORD				dwTimeout)
{
	return Receive((CIObject**)NULL, dwTimeout);
}


///////////////////////////////////////////////////////////////////////
// Delete (protected)

void CIObjectMbx::	Delete(void)
{
	// get rid of the RT object mailbox
	if (m_hRTObject != NULL_RTHANDLE)
		if (::DeleteRtMailbox(m_hRTObject))
			m_status= E_OK;
		else
			m_status= ::GetLastRtError();
	else
		m_status	= E_PARAM;

	// do base class deletion
	CIObject::Delete();
}
