///////////////////////////////////////////////////////////////////////
// implementation of the CIPIntHandler class
// last updated: 23Oct07
// Copyright(C) 2007 Micronet Co. All Rights Reserved.


#include "rtpp.h"

#define IS_MSI_LEVEL(lvl)	(((lvl) & 0xe000) == 0x6000)

///////////////////////////////////////////////////////////////////////
// constructor

CIPIntHandler::		CIPIntHandler(void)

	: CIObject(CITYPE_PINTHANDLER)
{
	m_byMaxInt		= 0;
}


///////////////////////////////////////////////////////////////////////
// destructor

CIPIntHandler::		~CIPIntHandler(void)
{
	// detach the element
	// (code copied here to reach own Delete function)
	if (InternalDetach())
	{
		Delete();
		m_hRTObject	= NULL_RTHANDLE;
	}
}


///////////////////////////////////////////////////////////////////////
// Create
// returns: zero or error code

DWORD CIPIntHandler::Create(
	WORD				wVendor,
	WORD				wDevice,
	WORD				wInstance)
{
	PVOID			par;

	Detach();
	m_cName[0]			= 0;
	SetRtProcessMaxPriority(NULL_RTHANDLE, 0);

	m_dev.wVendorId		= wVendor;
	m_dev.wDeviceId		= wDevice;
	m_dev.wDeviceIndex	= wInstance;
    if (!PciFindDevice(&m_dev))
		Fail("Cannot find PCI device (%x, %x, %x)", wVendor, wDevice, wInstance);

    // bring the device out of low power state, enable memory and I/O access
    PciEnableDevice(&m_dev);
 
    // Set bus master capability, if required
    PciSetMaster(&m_dev);
 
    // Find and map a memory base address in BAR0
    if (m_dev.IoSpace[0].flags & IOSPACE_MEM)
        m_regs			= (DWORD*)MapRtPhysicalMemory(m_dev.IoSpace[0].start, m_dev.IoSpace[0].size);
 
    // find the I/O base address in BAR1
    if (m_dev.IoSpace[1].flags & IOSPACE_IO)
        m_iobase		= (WORD)m_dev.IoSpace[1].start;
 
    // if the device is MSI-capable, set up the interrupt handler accordingly
    if (m_dev.MsiOffset != 0) {
        m_wLevel		= MSI_LEVEL;
        m_msi.PciAddress= MKPCIADDR((&m_dev));
		m_msi.ReservedZero = 0;
        m_msi.Param		= this;
		par				= &m_msi;
    }
    else {
        m_wLevel		= PciGetInterruptLevel(&m_dev);
        if (BAD_LEVEL == m_wLevel)
			Fail("Cannot find interrupt for PCI device (%x, %x, %x)", wVendor, wDevice, wInstance);
		m_wLevel       |= SHARED_LEVEL;
		par				= this;
	}

	// attempt to set up interrupt handling
	m_wLevel			= ::SetRtInterruptHandlerEx(m_wLevel, m_byMaxInt,
							(LPPROC)SharedInterruptHandler, par);
	if (BAD_LEVEL == m_wLevel)
		Fail("Cannot set interrupt for PCI device (%x, %x, %x)", wVendor, wDevice, wInstance);

	// change handle so it looks like a real object
	m_status		= E_OK;
	m_hRTObject		= BAD_RTHANDLE;
	m_bOwner		= TRUE;

	return 0;
}


///////////////////////////////////////////////////////////////////////
// Delete (protected)

void CIPIntHandler::Delete(void)
{
	if (m_byMaxInt)
		m_status	= E_OK;
	else if (m_hRTObject != NULL_RTHANDLE)
		if (::ResetRtInterruptHandler(m_wLevel))
			m_status= E_OK;
		else
			m_status= ::GetLastRtError();
	else
		m_status	= E_PARAM;

	CIObject::Delete();
}


///////////////////////////////////////////////////////////////////////
// Enable
// returns: zero or error code

DWORD CIPIntHandler::	Enable(
	BOOL				bState)
{
	BOOL			bResult;

	if (bState)
		bResult		= ::EnableRtInterrupt(m_wLevel);
	else
		bResult		= ::DisableRtInterrupt(m_wLevel);
	if (bResult)
		m_status	= ::GetLastRtError();
	else
		m_status	= E_OK;
	return m_status;
}


///////////////////////////////////////////////////////////////////////
// OnInterrupt
// This should contain the code to check the device, get status, etc. 
// Should return TRUE if the interrupt was for this handler, or
// FALSE if not. The FALSE return is most applicable to shared 
// handlers. 
BOOL CIPIntHandler::OnInterrupt(void)
{
	if (m_byMaxInt)
		return m_pThread->OnInterrupt();
	return TRUE;
}


///////////////////////////////////////////////////////////////////////
// InterruptHandler (static, generic shared interrupt handler)
// this handler assumes a local variable, and a signal for every interrupt
// when overruling this handler, please read all rules on writing handlers

__INTERRUPT void CIPIntHandler::
					SharedInterruptHandler(
	WORD			wCSRA,
	WORD			wLevel,
	LPVOID			pv)
{
	CIPIntHandler *	pHandler;

	__SHARED_INTERRUPT_PROLOG();

	// find the object and call the OnInterrupt function
	if (IS_MSI_LEVEL(wLevel))
		pHandler =	(CIPIntHandler*)((MSI_PARAM*)pv)->Param;
	else
		pHandler =	(CIPIntHandler*)pv;

	// Handle the interrupt
	if (pHandler) 
	{
		if (pHandler->OnInterrupt())
		{		
			// complete the interrupt
			if (pHandler->m_byMaxInt)
				SignalRtInterruptThread(wLevel);
		}
	}

	__SHARED_INTERRUPT_RETURN();
}
