///////////////////////////////////////////////////////////////////////
// implementation of the CIPIntThread class
// last updated: 23Oct07
// Copyright(C) 2007 Micronet Co. All Rights Reserved.


#include "rtpp.h"


///////////////////////////////////////////////////////////////////////
// constructor

CIPIntThread::		CIPIntThread(void)

	: CIThread()
{
}


///////////////////////////////////////////////////////////////////////
// destructor

CIPIntThread::		~CIPIntThread(void)
{
	// detach the element
	// (code copied here to reach own Delete function)
	if (InternalDetach())
	{
		Delete();
		m_hRTObject	= NULL_RTHANDLE;
	}
}


///////////////////////////////////////////////////////////////////////
// Create
// returns: zero or error code

DWORD CIPIntThread::	Create(
	char *				pszName,
	WORD				wVendor,
	WORD				wDevice,
	WORD				wInstance,
	BYTE				byMaxInt,
	DWORD				dwMilliseconds,
	DWORD				dwStack)
{
	m_handler.m_pThread	= this;
	m_handler.m_byMaxInt= byMaxInt;
	m_wVendor			= wVendor;
	m_wDevice			= wDevice;
	m_wInstance			= wInstance;
	m_dwMilliseconds	= dwMilliseconds;

	// the remainder is the same as for CIThread
	return CIThread::Create(pszName, 0, dwStack);
}


///////////////////////////////////////////////////////////////////////
// Delete (protected)

void CIPIntThread::	Delete(void)
{
	RTHANDLE		hTemp;


	m_handler.Delete();

	// since we may not come back from ResetRtInterruptHandler
	// make sure that the object is correct
	hTemp			= m_hRTObject;
	m_hRTObject		= NULL_RTHANDLE;
	if (hTemp != NULL_RTHANDLE)
		if (::ResetRtInterruptHandler(m_handler.m_wLevel))
			m_status= E_OK;
		else {
			m_status= ::GetLastRtError();
			DeleteRtThread(hTemp);
		}
	else
		m_status	= E_PARAM;

	CIObject::Delete();
}


///////////////////////////////////////////////////////////////////////
// InitInstance
// returns: TRUE if successful, FALSE if app should terminate

BOOL CIPIntThread::	InitInstance(void)
{
	m_status		= m_handler.Create(m_wVendor, m_wDevice, m_wInstance);
	return (E_OK == m_status);
}


///////////////////////////////////////////////////////////////////////
// DoForever
// returns: TRUE if loop should continue, FALSE if thread should terminate

BOOL CIPIntThread::	DoForever(void)
{
	// try to receive a signal
	if (!::WaitForRtInterrupt(m_handler.m_wLevel, m_dwMilliseconds))
	{
		// error: if it was a timeout, do normal processing
		m_status	= ::GetLastRtError();
		if (m_status == E_TIME)
			return OnSignal(FALSE);
		else
			return FALSE;
	}
	else
		return OnSignal(TRUE);
}


///////////////////////////////////////////////////////////////////////
// OnInterrupt
// This should contain the code to check the device, get status, etc. 
// Should return TRUE if the interrupt was for this handler, or
// FALSE if not. The FALSE return is most applicable to shared 
// handlers. 
BOOL CIPIntThread::	OnInterrupt(void)
{
	return TRUE;
}


///////////////////////////////////////////////////////////////////////
BOOL CIPIntThread::	OnSignal(
	BOOL				bSignal)
{
	return TRUE;
}


///////////////////////////////////////////////////////////////////////
// Enable
// returns: zero or error code

DWORD CIPIntThread::	Enable(
	BOOL				bState)
{
	return m_handler.Enable(bState);
}
