///////////////////////////////////////////////////////////////////////
// implementation of the CISemaphore class
// last updated: 28oct98
// Copyright(C) 1998 Micronet Co. All Rights Reserved.

#include "rtpp.h"


///////////////////////////////////////////////////////////////////////
// constructor

CISemaphore::		CISemaphore(void)

	: CIObject(CITYPE_SEMAPHORE)
{
}


///////////////////////////////////////////////////////////////////////
// destructor

CISemaphore::		~CISemaphore(void)
{
	// detach the RT object
	// (code copied here to reach own Delete function)
	if (InternalDetach())
		Delete();
}


///////////////////////////////////////////////////////////////////////
// Create
// returns: zero or error code

DWORD CISemaphore::	Create(
	char *				pszName,
	WORD				wMax,
	BOOL				bPrioQ)
{
	RTHANDLE		h;

	// detach from RT object
	Detach();

	// create the semaphore
	h				= ::CreateRtSemaphore(0, wMax,
						(bPrioQ ? PRIORITY_QUEUING : FIFO_QUEUING));
	if (h == BAD_RTHANDLE)
		return (m_status = ::GetLastRtError());

	// attach the RT semaphore object (as owner)
	m_status		= E_OK;
	InternalAttach(pszName, h, TRUE);

	return m_status;
}


///////////////////////////////////////////////////////////////////////
// Release
// returns: zero or error code

DWORD CISemaphore::	Release(
	WORD				wUnits)
{
	if (m_hRTObject == NULL_RTHANDLE)
		m_status	= E_EXIST;

	else if (::ReleaseRtSemaphore(m_hRTObject, wUnits))
		m_status	= E_OK;

	else
		m_status	= ::GetLastRtError();

	return m_status;
}


///////////////////////////////////////////////////////////////////////
// Wait
// returns: number of units remaining, or WAIT_FAILED

DWORD CISemaphore::	Wait(
	WORD				wUnits,
	DWORD				dwTimeout)
{
	DWORD			dwResult;

	dwResult		= WAIT_FAILED;
	if (m_hRTObject == NULL_RTHANDLE)
		m_status	= E_EXIST;

	else
	{
		dwResult	= ::WaitForRtSemaphore(m_hRTObject, wUnits, dwTimeout);
		if (dwResult == WAIT_FAILED)
			m_status= ::GetLastRtError();
	}

	return dwResult;
}


///////////////////////////////////////////////////////////////////////
// Delete (protected)

void CISemaphore::	Delete(void)
{
	// get rid of the RT semahore object
	if (m_hRTObject != NULL_RTHANDLE)
		if (::DeleteRtSemaphore(m_hRTObject))
			m_status= E_OK;
		else
			m_status= ::GetLastRtError();
	else
		m_status	= E_PARAM;

	// do base class deletion
	CIObject::Delete();
}
