///////////////////////////////////////////////////////////////////////
// implementation of the CIThread class
// last updated: 28oct98
// Copyright(C) 1998 Micronet Co. All Rights Reserved.

#include "rtpp.h"


///////////////////////////////////////////////////////////////////////
// constructor

CIThread::			CIThread(void)

	: CIObject(CITYPE_THREAD)
{
}


///////////////////////////////////////////////////////////////////////
// destructor

CIThread::			~CIThread(void)
{
	// detach the RT object
	// (code copied here to reach own Delete function)
	if (InternalDetach())
	{
		Delete();
		m_hRTObject	= NULL_RTHANDLE;
	}
}


///////////////////////////////////////////////////////////////////////
// Create
// returns: zero or error code

DWORD CIThread::	Create(
	char *				pszName,
	BYTE				byPrio,
	DWORD				dwStack,
	BOOL				bGo)
{
	RTHANDLE		h;

	// detach from RT object
	Detach();

	// prepare member variables for pickup
	m_bGo			= bGo;
	
	// create the thread at the best priority for this process
	h				= ::CreateRtThread(byPrio,
						(LPPROC)GenericStart,
						dwStack,
						(PVOID)this);
	if (h == BAD_RTHANDLE)
		m_status	= ::GetLastRtError();
	else
	{
		m_status	= E_OK;
		InternalAttach(pszName, h, TRUE);
	}

	return m_status;
}


///////////////////////////////////////////////////////////////////////
// Delete (protected)

void CIThread::		Delete(void)
{
	RTHANDLE		hTemp;

	// since we may not come back from DeleteRtThread,
	// make sure that the object is correct
	hTemp			= m_hRTObject;
	m_hRTObject		= NULL_RTHANDLE;
	if (hTemp != NULL_RTHANDLE)
		if (::DeleteRtThread(hTemp))
			m_status= E_OK;
		else
			m_status= ::GetLastRtError();
	else
		m_status	= E_PARAM;

	// do base class deletion
	CIObject::Delete();
}


///////////////////////////////////////////////////////////////////////
// SuspendThread
// returns: zero or error code

DWORD CIThread::	SuspendThread(void)
{
	if (m_hRTObject == NULL_RTHANDLE)
		m_status	= E_EXIST;
	else if (!::SuspendRtThread(m_hRTObject))
		m_status	= ::GetLastRtError();
	else
		m_status	= E_OK;
	return m_status;
}


///////////////////////////////////////////////////////////////////////
// ResumeThread
// returns: zero or error code

DWORD CIThread::	ResumeThread(void)
{
	if (m_hRTObject == NULL_RTHANDLE)
		m_status	= E_EXIST;
	if (!::ResumeRtThread(m_hRTObject))
		m_status	= ::GetLastRtError();
	else
		m_status	= E_OK;
	return m_status;
}


///////////////////////////////////////////////////////////////////////
// GetThreadPriority
// returns: priority or BAD_PRIORITY

BYTE CIThread::		GetThreadPriority(void)
{
	BYTE			byPrio;

	byPrio			= BAD_PRIORITY;

	if (m_hRTObject == NULL_RTHANDLE)
		m_status	= E_EXIST;
	else
	{
		byPrio		= ::GetRtThreadPriority(m_hRTObject);
		if (byPrio == BAD_PRIORITY)
			m_status	= ::GetLastRtError();
		else
			m_status	= E_OK;
	}
	return byPrio;
}


///////////////////////////////////////////////////////////////////////
// SetThreadPiority
// returns: zero or error code

DWORD CIThread::	SetThreadPriority(
	BYTE				byPrio)
{
	if (m_hRTObject == NULL_RTHANDLE)
		m_status	= E_EXIST;
	else
		m_status	= ::SetRtThreadPriority(m_hRTObject, byPrio);
	return m_status;
}


///////////////////////////////////////////////////////////////////////
// Sleep (static)

DWORD CIThread::	Sleep(
	DWORD				dwTime)
{
	DWORD			dwStatus;

	if (::RtSleep(dwTime))
		dwStatus	= E_OK;
	else
		dwStatus	= ::GetLastRtError();
	return dwStatus;
}


///////////////////////////////////////////////////////////////////////
// GenericStart (static, protected)

void CIThread::		GenericStart(
	PVOID				pV)
{
	((CIThread*)pV)->GenericBody();
}


///////////////////////////////////////////////////////////////////////
// GenericBody (protected)

void CIThread::		GenericBody(void)
{
	if (!m_bGo)
		SuspendThread();

	if (InitInstance())
	{
		while (DoForever())
			;
	}
	
	ExitInstance();

	Detach();
}


///////////////////////////////////////////////////////////////////////

BOOL CIThread::		InitInstance(void)
{
	return TRUE;
}


///////////////////////////////////////////////////////////////////////

BOOL CIThread::		DoForever(void)
{
	return FALSE;
}


///////////////////////////////////////////////////////////////////////

int CIThread::		ExitInstance(void)
{
	return 0;
}
