///////////////////////////////////////////////////////////////////////
// definitions of the INtime classes
// last updated: 23Jul2009
// Copyright(C) 2009 Micronet Co. All Rights Reserved.

#ifndef _RTPP_h_
#define _RTPP_h_

#include <rt.h>
#include <pcibus.h>

#pragma pack(push,enter_rtpp)
#pragma pack(1)

#ifdef __C99__
#  ifdef RTCLASS_EXPORTS
#    define RTCLASS_SPEC __declspec(dllexport)
#  else
#    pragma comment(linker, "/include:_main") // force linker to use rtpp400[d].lib:main
#    define RTCLASS_SPEC //__declspec(dllimport)
#  endif
#else
#  define RTCLASS_SPEC
#endif


///////////////////////////////////////////////////////////////////////
// constants

#define CITYPE_INVALID		0
#define CITYPE_DATAMBX		1
#define CITYPE_OBJECTMBX	2
#define CITYPE_LLMBX		3
#define CITYPE_SEMAPHORE	4
#define CITYPE_LLSEMAPHORE	5
#define CITYPE_REGION		6
#define CITYPE_MEMORY		7
#define CITYPE_PORT			8
#define CITYPE_INTHANDLER	9
#define CITYPE_THREAD		10
#define CITYPE_PINTHANDLER	9


///////////////////////////////////////////////////////////////////////
// macros for debugging

#ifndef ASSERT
	#ifdef _DEBUG
		// ASSERT calls Fail when its argument is FALSE
		#define ASSERT(f) \
			{ if (!(f))\
				{ \
					static char temp[256]; \
					sprintf(temp, "File %s line %d assertion %s failed", \
						__FILE__, __LINE__, #f); \
					Fail(temp); \
				} \
			}
	#else
		#define ASSERT(f)
	#endif
#endif


///////////////////////////////////////////////////////////////////////
// macros for wizard generated source

#ifndef VS7_CPP

// VS.6 Wizard macros

#define	DECLARE_MEMORY(name) \
	CIMemory		m_h##name;

#define	DECLARE_INTERRUPT(level) \
	CII##level		m_I##level;

#define	DECLARE_MBXSEM(name) \
	CIS##name		m_S##name;

#define	DECLARE_POLL(name) \
	CIPoll##name	m_Poll##name;

#define	DECLARE_CLIENT(name) \
	CIClient##name	m_Client##name;

#define	CREATE_MEMORY(name,size) \
	m_h##name.Create(#name, size);

#define	CREATE_MBXSEM_DMBX(name,prio,stack) \
	m_S##name.Create("", #name, prio, stack);

#define	CREATE_MBXSEM_OMBX(name,prio,stack) \
	m_S##name.Create("", #name, prio, stack);

#define	CREATE_MBXSEM_SEM(name,prio,stack) \
	m_S##name.Create("", #name, 1, prio, stack);

#define	CREATE_MBXSEM_UMBX(name, size, max, prio, stack) \
	name.Create(size, max); \
	m_S##name.Create("", &name, size, prio, stack);

#define	CREATE_MBXSEM_USEM(name, prio, stack) \
	name.Create(); \
	m_S##name.Create("", &name, prio, stack);

#define	CREATE_INTERRUPT_THREAD(name, level, sigmax, timeout, stack, shared) \
	m_I##name.Create("", level, sigmax, timeout, stack, shared);

#define	CREATE_INTERRUPT_HANDLER(name, level, shared) \
	m_I##name.Create(level, shared);

#define	CREATE_INTERRUPT_UMBX(name, level, mbxname, shared) \
	m_I##name.Create(level, &mbxname, shared);

#define	CREATE_INTERRUPT_USEM(name, level, semname, shared) \
	m_I##name.Create(level, &semname, shared);

#define CREATE_POLL_MSEC(name,period,prio,stack) \
	m_Poll##name.Create("Poll" #name, period, prio, stack);

#define CREATE_POLL_USEC(name,period,prio,stack) \
	m_Poll##name.Create("Poll" #name, period, prio, stack);

#define CREATE_CLIENT(name,prio,stack) \
	m_Client##name.Create("Client" #name, prio, stack);

#else

// VS.7 wizard macros

#define	DECLARE_MEMORY(name) \
	CIMemory		m_h##name;

#define	DECLARE_INTERRUPT(level) \
	CII##level		m_I##level;

#define	DECLARE_MBXSEM(name) \
	CI##name		m_##name;

#define	DECLARE_POLL(name) \
	CI##name	m_##name;

#define	DECLARE_CLIENT(name) \
	CI##name	m_##name;

#define	CREATE_MEMORY(name,size) \
	m_h##name.Create(#name, size);

#define	CREATE_MBXSEM_DMBX(name,prio,stack) \
	m_##name.Create("", #name, prio, stack);

#define	CREATE_MBXSEM_OMBX(name,prio,stack) \
	m_##name.Create("", #name, prio, stack);

#define	CREATE_MBXSEM_SEM(name,prio,stack) \
	m_##name.Create("", #name, 1, prio, stack);

#define	CREATE_MBXSEM_UMBX(name, size, max, prio, stack) \
	name.Create(size, max); \
	m_##name.Create("", &name, size, prio, stack);

#define	CREATE_MBXSEM_USEM(name, prio, stack) \
	name.Create(); \
	m_##name.Create("", &name, prio, stack);

#define	CREATE_INTERRUPT_THREAD(name, level, sigmax, timeout, stack, shared) \
	m_I##name.Create("", level, sigmax, timeout, stack, shared);

#define	CREATE_INTERRUPT_HANDLER(name, level, shared) \
	m_I##name.Create(level, shared);

#define	CREATE_INTERRUPT_UMBX(name, level, mbxname, shared) \
	m_I##name.Create(level, &mbxname, shared);

#define	CREATE_INTERRUPT_USEM(name, level, semname, shared) \
	m_I##name.Create(level, &semname, shared);

#define	CREATE_PINTERRUPT_THREAD(name, ven, dev, inst, sigmax, timeout, stack) \
	m_I##name.Create("", ven, dev, inst, sigmax, timeout, stack);

#define	CREATE_PINTERRUPT_HANDLER(name, ven, dev, inst) \
	m_I##name.Create(ven, dev, inst);

#define	CREATE_PINTERRUPT_UMBX(name, ven, dev, inst, mbxname) \
	m_I##name.Create(ven, dev, inst, &mbxname);

#define	CREATE_PINTERRUPT_USEM(name, ven, dev, inst, semname) \
	m_I##name.Create(ven, dev, inst, &semname);

#define CREATE_POLL_MSEC(name,period,prio,stack) \
	m_##name.Create("Poll" #name, period, prio, stack);

#define CREATE_POLL_USEC(name,period,prio,stack) \
	m_##name.Create("Poll" #name, period, prio, stack);

#define CREATE_CLIENT(name,prio,stack) \
	m_##name.Create("Client" #name, prio, stack);

#endif


///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
// definition of the base class: CIObject

class RTCLASS_SPEC
					CIObject
{
public:
	// constructor and destructor for derived classes only
					CIObject(WORD wType);
	virtual			~CIObject(void);

public:
	// member functions for general use
#if defined(_DEBUG) || defined(__C99__)
	DWORD			GetLastError(void) const;
	RTHANDLE		GetHandle(void) const;
	WORD			GetType(void) const;
#else
	DWORD			GetLastError(void) const
						{ return m_status; }
	RTHANDLE		GetHandle(void) const
						{ return m_hRTObject; }
	WORD			GetType(void) const
						{ return m_wType; }
#endif

	// member functions that may be overruled
	virtual DWORD	Lookup (char * pszName, DWORD dwTimeout = WAIT_FOREVER);
	virtual DWORD	Attach(RTHANDLE h);
	virtual void	Detach(void);

	// operators (may be overruled)
	virtual CIObject &
					operator = (CIObject & src);
	virtual BOOL	operator == (CIObject & op2);
	virtual BOOL	operator != (CIObject & op2);

	// class functions
	static void		Fail(char * pszMessage, ...);

protected:
	// internal functions that may be overruled
	virtual void	Delete(void);
	virtual	void	Remove(void);
	virtual DWORD	Catalog(void);
	virtual DWORD	Uncatalog(void);

	// member functions for internal use only
	BOOL			ParseName(char * pszFullName,
						RTHANDLE * hProcess, char * pszName);
	void			InternalAttach(char * pszName,
						RTHANDLE h, BOOL bOwner);
	BOOL			InternalDetach(void);
	WORD			InternalGetType(RTHANDLE h);

	// member variables
	WORD			m_wType;
	RTHANDLE		m_hRTObject;
	DWORD			m_status;
	BOOL			m_bOwner;
	RTHANDLE		m_hCatProcess;
	char			m_cName[13];
};


///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
// definition of the CIDataMbx class

class RTCLASS_SPEC
					CIDataMbx : public CIObject
{
public:
	// constructor and destructor
					CIDataMbx(void);
					~CIDataMbx(void);

	// member functions for general use
	virtual DWORD	Create(char * pszName, BOOL bPrioQ = FALSE);
	virtual DWORD	Send(PVOID pv, WORD wLen);
	virtual WORD	Receive(PVOID pv, WORD wMax, DWORD dwTimeout = WAIT_FOREVER);

protected:
	// member functions for internal use only
	virtual void	Delete(void);
};


///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
// definition of the CIObjectMbx class

class RTCLASS_SPEC
					CIObjectMbx : public CIObject
{
public:
	// constructor and destructor
					CIObjectMbx(void);
					~CIObjectMbx(void);

	// member functions for general use
	virtual DWORD	Create(char * pszName, BOOL bPrioQ = FALSE);
	virtual DWORD	Send(CIObject * pObject,
						CIObject * pReply=(CIObject*)NULL);
	virtual CIObject *
					Receive(CIObject ** pReply=(CIObject **)NULL,
						DWORD dwTimeout = WAIT_FOREVER);
	virtual CIObject *
					Receive(DWORD dwTimeout = WAIT_FOREVER);

protected:
	// member functions for internal use only
	void			Delete(void);
};


///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
// definition of the CISemaphore class

class RTCLASS_SPEC
					CISemaphore : public CIObject
{
public:
	// constructor and destructor
					CISemaphore(void);
					~CISemaphore(void);

	// member functions for general use
	virtual DWORD	Create(char * pszName, WORD wMax=1,
						BOOL bPrioQ = FALSE);
	virtual DWORD	Release(WORD wUnits=1);
	virtual DWORD	Wait(WORD wUnits=1, DWORD dwTimeout = WAIT_FOREVER);

protected:
	// member functions for internal use only
	virtual void	Delete(void);
};


///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
// definition of the CIRegion class

class RTCLASS_SPEC
					CIRegion : public CIObject
{
public:
	// constructor and destructor
					CIRegion(void);
					~CIRegion(void);

	// member functions for general use
	virtual DWORD	Create(char * pszName, BOOL bPrioQ = FALSE);
	virtual DWORD	Release(void);
	virtual DWORD	Wait(DWORD dwTimeout = WAIT_FOREVER);

protected:
	// member functions for internal use only
	virtual void	Delete(void);
};


///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
// definition of the CIMemory class

class RTCLASS_SPEC
					CIMemory : public CIObject
{
public:
	// constructor and destructor
					CIMemory(void);
					~CIMemory(void);

	// member functions for general use
	virtual DWORD	Create(char * pszName, DWORD dwLen, PVOID pAddr=NULL);
 	virtual PVOID	GetPointer(void);
	virtual DWORD	GetSize(void);
	virtual CIMemory &
					operator = (CIMemory & src);

	// overruled to append mapping
	virtual DWORD	Lookup (char * pszName, DWORD dwTimeout = WAIT_FOREVER);
	virtual DWORD	Attach(RTHANDLE h);

protected:
	// member functions for internal use only
	virtual void	Delete(void);
	virtual void	Remove(void);
	DWORD			Map(void);

	// member variables
	PVOID			m_p;
	DWORD			m_dwLen;
	BOOL			m_bAllocated;
};


/// ///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
// definition of the CILLMbx class

class RTCLASS_SPEC
					CILLMbx : public CIObject
{
public:
	// constructor and destructor
					CILLMbx(void);
					~CILLMbx(void);

	// member functions for general use
	virtual DWORD	Create(DWORD dwMsgSize, DWORD dwQueueLength,
						BOOL bPrioQ = FALSE, BOOL bPrioSlot = FALSE);
	virtual DWORD	Send(PVOID pv, DWORD dwLen);
	virtual DWORD	SendPriority(PVOID pv, DWORD dwLen);
	virtual DWORD	Receive(PVOID pv, DWORD dwMax, DWORD dwUsecs = WAIT_FOREVER);

protected:
	// member functions for internal use only
	virtual void	Delete(void);

	// member variables
	KNHANDLE		m_hLLObject;
	KNPTR			m_pArea;
	DWORD			m_dwMsgSize;
};


///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
// definition of the CILLSemaphore class

class RTCLASS_SPEC
					CILLSemaphore : public CIObject
{
public:
	// constructor and destructor
					CILLSemaphore(void);
					~CILLSemaphore(void);

	// member functions for general use
	virtual DWORD	Create(DWORD dwFlags = KN_FIFO_QUEUING);
	virtual DWORD	Release(void);
	virtual DWORD	Wait(DWORD dwUsecs = WAIT_FOREVER);

protected:
	// member functions for internal use only
	virtual void	Delete(void);

	// member variables
	KNHANDLE		m_hLLObject;
	DWORD			m_Area[KN_SEMAPHORE_SIZE/4];
};


///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
// definition of the CIThread class

class RTCLASS_SPEC
					CIThread : public CIObject
{
public:
	// constructor and destructor
					CIThread(void);
	virtual			~CIThread(void);

	// member functions for general use
	virtual DWORD	Create(char * pszName, BYTE byPrio, DWORD dwStack=4096,
						BOOL bGo=TRUE);
	DWORD			SuspendThread(void);
	DWORD			ResumeThread(void);
	BYTE			GetThreadPriority(void);
	DWORD			SetThreadPriority(BYTE byPrio);

	static DWORD	Sleep(DWORD dwTime);

	// member functions that may be overruled
	virtual BOOL	InitInstance(void);
	virtual BOOL	DoForever(void);
	virtual int		ExitInstance(void);

protected:
	// member functions for internal use only
	virtual void	Delete(void);
	static void		GenericStart(PVOID pV);
	void			GenericBody(void);

	// member variables only used for thread start
	BOOL			m_bGo;
};


///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
// definition of the CIDataMbxThread class

class RTCLASS_SPEC
					CIDataMbxThread : public CIThread
{
public:
	// constructor and destructor
					CIDataMbxThread(void);

	// member functions for general use
	virtual DWORD	Create(char * pszName, char * pszMbx,
						BYTE byPrio, DWORD dwStack=4096, BOOL bGo=TRUE);

	// member functions that may be overruled
	virtual BOOL	DoForever(void);
	virtual int		ExitInstance(void);
	virtual BOOL	OnReceive(BYTE * pBuffer, WORD wLen);

protected:
	// member variables
	CIDataMbx		m_Mbx;
};


///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
// definition of the CIObjectMbxThread class

class RTCLASS_SPEC
					CIObjectMbxThread : public CIThread
{
public:
	// constructor and destructor
					CIObjectMbxThread(void);
	// member functions for general use
	virtual DWORD	Create(char * pszName, char * pszMbx,
						BYTE byPrio, DWORD dwStack=4096, BOOL bGo=TRUE);

	// member functions that may be overruled
	virtual BOOL	DoForever(void);
	virtual int		ExitInstance(void);
	virtual BOOL	OnReceive(CIObject * pObject, CIObject * pReply);

protected:
	// member variables
	CIObjectMbx		m_Mbx;
};


///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
// definition of the CILLMbxThread class

class RTCLASS_SPEC
					CILLMbxThread : public CIThread
{
public:
	// constructor and destructor
					CILLMbxThread(void);

	// member functions for general use
	virtual DWORD	Create(char * pszName,
						CILLMbx * pMbx, DWORD dwMsgSize,
						BYTE byPrio, DWORD dwStack=4096, BOOL bGo=TRUE);

	// member functions that may be overruled
	virtual BOOL	DoForever(void);
	virtual int		ExitInstance(void);
	virtual BOOL	OnReceive(BYTE * pBuffer, DWORD dwLen);

protected:
	// member variables
	CILLMbx *		m_pMbx;
	BYTE *			m_pBuffer;
	DWORD			m_dwLen;
};


///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
// definition of the CISemaphoreThread class

class RTCLASS_SPEC
					CISemaphoreThread : public CIThread
{
public:
	// constructor and destructor
					CISemaphoreThread(void);

	// member functions for general use
	virtual DWORD	Create(char * pszName, char * pszSema, WORD wMax,
						BYTE byPrio, DWORD dwStack=4096, BOOL bGo=TRUE);

	// member functions that may be overruled
	virtual BOOL	DoForever(void);
	virtual int		ExitInstance(void);
	virtual BOOL	OnReceive(void);

protected:
	// member variables
	CISemaphore		m_Semaphore;
};


///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
// definition of the CILLSemaphoreThread class

class RTCLASS_SPEC
					CILLSemaphoreThread : public CIThread
{
public:
	// constructor and destructor
					CILLSemaphoreThread(void);

	// member functions for general use
	virtual DWORD	Create(char * pszName, CILLSemaphore * pSem,
						BYTE byPrio, DWORD dwStack=4096, BOOL bGo=TRUE);

	// member functions that may be overruled
	virtual BOOL	DoForever(void);
	virtual BOOL	OnReceive(void);

protected:
	// member variables
	CILLSemaphore *	m_pSemaphore;
};


///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
// definition of the CIPollThread class

class RTCLASS_SPEC
					CIPollThread : public CIThread
{
public:
	// constructor and destructor
					CIPollThread(void);

	// member functions for general use
	virtual DWORD	Create(char * pszName, DWORD dwTime, BYTE byPrio,
						DWORD dwStack=4096, BOOL bGo=TRUE);

	virtual void	SetPoll(DWORD dwTime);

	// member functions that may be overruled
	virtual BOOL	DoForever(void);
	virtual BOOL	OnPoll(void);

protected:
	// member variables
	DWORD			m_dwTime;
};


///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
// definition of the CILLPollThread class

class RTCLASS_SPEC
					CILLPollThread : public CIThread
{
public:
	// constructor and destructor
					CILLPollThread(void);
					~CILLPollThread(void);

	// member functions for general use
	virtual DWORD	Create(char * pszName, DWORD dwUsecs, BYTE byPrio,
						DWORD dwStack=4096, BOOL bGo=TRUE);
	void			SetPoll(DWORD dwUsecs);

	// member functions that may be overruled
	virtual BOOL	DoForever(void);
	virtual BOOL	OnPoll(void);

protected:
	// member variables
	DWORD			m_dwTicks;
	KNHANDLE		m_hAlarm;
	DWORD			m_dwAlarm[KN_ALARM_EVENT_SIZE/4];
	BOOL			m_bChanged;
};


///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
// definition of the CILLPollThread2 class

class RTCLASS_SPEC
					CILLPollThread2 : public CIThread
{
public:
	// constructor and destructor
					CILLPollThread2(void);
					~CILLPollThread2(void);

	// member functions for general use
	virtual DWORD	Create(char * pszName, DWORD dwUsecs, BYTE byPrio,
						DWORD dwStack=4096, BOOL bGo=TRUE);
	void			SetPoll(DWORD dwUsecs);

	// member functions that may be overruled
	virtual BOOL	DoForever(void);
	virtual BOOL	OnPoll(void);

protected:
	// member variables
	DWORD			m_dwUsecs;
	RTHANDLE		m_hAlarm;
	BOOL			m_bChanged;
};


///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
// definition of the CIApp class

class RTCLASS_SPEC
					CIApp : public CIThread
{
public:
	// constructor and destructor
					CIApp(char * pszName);
	virtual			~CIApp(void);

	// member functions that can be overruled
	virtual BOOL	InitInstance(void);
	virtual BOOL	DoForever(void);
	virtual int		ExitInstance(void);
	virtual BOOL	OnNotify(LPEVENTINFO pEventInfo);

	// member functions overruled to force failure
	virtual DWORD	Lookup (char * pszName, DWORD dwTimeout = WAIT_FOREVER);
	virtual DWORD	Attach(RTHANDLE h);
	virtual void	Detach(void);

	// member functions overruled for different behaviour
	virtual DWORD	Create(int argc, char ** argv);
	virtual DWORD	Catalog(void);
	virtual DWORD	Uncatalog(void);

	// member functions for general use
	DWORD			UsecsToKticks(DWORD dwUsecs);
	static WORD		GetPciInterruptLevel(WORD wPciVendorId, WORD wPciDeviceId);
#if defined(_DEBUG) || defined(__C99__)
	static CIApp *	GetApp(void);
#else
	static CIApp *	GetApp(void)
						{ return m_pApp; }
#endif
	static BOOL		IsMain(void)
						{ return m_pApp->m_main == GetRtThreadHandles(THIS_THREAD); }
protected:
	// member functions for internal use only
	virtual void	Delete(void);

	// member variables
	int				m_iArg;
	char **			m_pArg;

	// class variables
	static CIApp *	m_pApp;
	RTHANDLE		m_main;

private:
	// member variables for internal use only
	BOOL			m_bCreated;
	DWORD			m_dwKtickInUsecs;
};


///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
// definition of the CIIntHandler class

class RTCLASS_SPEC
					CIIntHandler;
typedef struct		{
	WORD				wLevel;
	CIIntHandler *		pHandler;
	}				INTHANDLER;

class				CIIntHandler : public CIObject
{
public:
    friend class    CIIntThread;

	// constructor and destructor
					CIIntHandler(void);
					~CIIntHandler(void);

	// member functions for general use
	virtual DWORD	Create(WORD wLevel, BOOL bShared=FALSE);

	// member functions that may be overruled
	virtual BOOL	OnInterrupt(void);
	virtual DWORD	Enable(BOOL bState=TRUE);

protected:
	// member functions for internal use only
	virtual void	Delete(void);
	static void		InterruptHandler(void);

	// member variables
	WORD			m_wLevel, m_wHardLevel;
	BYTE			m_byMaxInt;
	CIIntThread*	m_pThread;
	BOOL			m_bShared; // not used anymore

	static INTHANDLER
					m_pObjects[16];
};


///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
// definition of the CIPIntHandler class

class RTCLASS_SPEC
					CIPIntHandler : public CIObject
{
public:
    friend class    CIPIntThread;

	// constructor and destructor
					CIPIntHandler(void);
					~CIPIntHandler(void);

	// member functions for general use
	virtual DWORD	Create(WORD wVendor, WORD wDevice, WORD wInstance);

	// member functions that may be overruled
	virtual BOOL	OnInterrupt(void);
	virtual DWORD	Enable(BOOL bState=TRUE);

protected:
	// member functions for internal use only
	virtual void	Delete(void);
	static void		SharedInterruptHandler(WORD wCSRA, WORD wLevel, LPVOID pv);

	// member variables
	WORD			m_wLevel;
	BYTE			m_byMaxInt;
	CIPIntThread*	m_pThread;
	PCIDEV			m_dev;
	WORD			m_iobase;
	DWORD*			m_regs;
	MSI_PARAM		m_msi;
};


///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
// definition of the CIIntThread class

class RTCLASS_SPEC
					CIIntThread : public CIThread
{
public:
	// constructor and destructor
					CIIntThread(void);
					~CIIntThread(void);

	// member functions for general use
	virtual DWORD	Create(char * pszName, WORD wLevel,
						BYTE byMaxInt=1, DWORD dwMilliseconds=WAIT_FOREVER,
						DWORD dwStack=4096, BOOL bShared=FALSE);

	// member functions that may be overruled
	virtual BOOL	InitInstance(void);
	virtual BOOL	DoForever(void);
	virtual BOOL	OnInterrupt(void);
	virtual BOOL	OnSignal(BOOL bSignal);
	virtual DWORD	Enable(BOOL bState=TRUE);

	// member functions for internal use only
	virtual void	Delete(void);

protected:
	// member variables
	WORD			m_wLevel;
	BOOL			m_bShared; // not used anymore
	DWORD			m_dwMilliseconds;
	CIIntHandler	m_handler;
};


///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
// definition of the CIPIntThread class

class RTCLASS_SPEC
					CIPIntThread : public CIThread
{
public:
	// constructor and destructor
					CIPIntThread(void);
					~CIPIntThread(void);

	// member functions for general use
	virtual DWORD	Create(char * pszName, WORD wVendor, WORD wDevice, WORD wInstance,
						BYTE byMaxInt=1, DWORD dwMilliseconds=WAIT_FOREVER,
						DWORD dwStack=4096);

	// member functions that may be overruled
	virtual BOOL	InitInstance(void);
	virtual BOOL	DoForever(void);
	virtual BOOL	OnInterrupt(void);
	virtual BOOL	OnSignal(BOOL bSignal);
	virtual DWORD	Enable(BOOL bState=TRUE);

	// member functions for internal use only
	virtual void	Delete(void);

protected:
	// member variables
	WORD			m_wVendor, m_wDevice, m_wInstance;
	DWORD			m_dwMilliseconds;
	CIPIntHandler	m_handler;
};


///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
// definition of the CILLIntMbxHandler class

class RTCLASS_SPEC
					CILLIntMbxHandler : public CIIntHandler
{
public:
	// constructor and destructor
					CILLIntMbxHandler(void);
					~CILLIntMbxHandler(void);

	// member functions for general use
	virtual DWORD	Create(WORD wLevel, CILLMbx * pMbx,
						BOOL bShared=FALSE);

	// member functions that may be overruled
	virtual void	Send(void);

	// overruled from baseclass
	virtual BOOL	OnInterrupt(void);

protected:
	// member variables
	CILLMbx *		m_pMbx;
};


///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
// definition of the CILLPIntMbxHandler class

class RTCLASS_SPEC
					CILLPIntMbxHandler : public CIPIntHandler
{
public:
	// constructor and destructor
					CILLPIntMbxHandler(void);
					~CILLPIntMbxHandler(void);

	// member functions for general use
	virtual DWORD	Create(WORD wVendor, WORD wDevice, WORD wInstance, CILLMbx * pMbx);

	// member functions that may be overruled
	virtual void	Send(void);

	// overruled from baseclass
	virtual BOOL	OnInterrupt(void);

protected:
	// member variables
	CILLMbx *		m_pMbx;
};


///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
// definition of the CILLIntSemaphoreHandler class

class RTCLASS_SPEC
					CILLIntSemaphoreHandler : public CIIntHandler
{
public:
	// constructor and destructor
					CILLIntSemaphoreHandler(void);
					~CILLIntSemaphoreHandler(void);

	// member functions for general use
	virtual DWORD	Create(WORD wLevel, CILLSemaphore * pSem,
						BOOL bShared);

	// member functions that may be overruled
	virtual void	Release(void);

	// overruled from baseclass
	virtual BOOL	OnInterrupt(void);

protected:
	// member variables
	CILLSemaphore *	m_pSem;
};


///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
// definition of the CILLPIntSemaphoreHandler class

class RTCLASS_SPEC
					CILLPIntSemaphoreHandler : public CIPIntHandler
{
public:
	// constructor and destructor
					CILLPIntSemaphoreHandler(void);
					~CILLPIntSemaphoreHandler(void);

	// member functions for general use
	virtual DWORD	Create(WORD wVendor, WORD wDevice, WORD wInstance, CILLSemaphore * pSem);

	// member functions that may be overruled
	virtual void	Release(void);

	// overruled from baseclass
	virtual BOOL	OnInterrupt(void);

protected:
	// member variables
	CILLSemaphore *	m_pSem;
};

#pragma pack(pop,enter_rtpp)

//#undef RTCLASS_SPEC

#endif // _RTPP_h_
