/*----------------------------------------------------------------

	ifcan (Interface PCI4853) INtime driver


			physical memory access helper  class


	   Copyright(C) programed by Kazumix ,Micronet 2016

------------------------------------------------------------------*/
#include "rtpp.h"
#include "PhysMemory.h"


CIPhysMemory::CIPhysMemory(void)
{
	m_status		= 0;
	m_p				= 0;
	m_dwPhysicalAddress	= 0;
	m_dwLen			= 0;
	m_bCreateOwner	= FALSE;
}

CIPhysMemory::CIPhysMemory(DWORD dwPhysAddr, DWORD dwLen)
{
	m_status		= 0;
	m_p				= 0;
	m_dwPhysicalAddress	= 0;
	m_dwLen			= 0;
	m_bCreateOwner	= FALSE;
	Create( dwPhysAddr ,dwLen );
}


CIPhysMemory::~CIPhysMemory(void)
{
	Delete();
}


///////////////////////////////////////////////////////////////////////
// Create
// returns: zero or error code

DWORD CIPhysMemory::Create(DWORD dwPhysAddr, DWORD dwLen)
{
	if( ! m_p )
	{
		m_dwPhysicalAddress	= dwPhysAddr;
		m_dwLen				= dwLen;

		m_p	= ::MapRtPhysicalMemory( m_dwPhysicalAddress ,m_dwLen );
		if( m_p == NULL )
			return (m_status = ::GetLastRtError());

		m_status		= E_OK;
		m_bCreateOwner	= TRUE;
		return m_status;
	}
	else{
		m_status		= E_CONTEXT;
		return m_status;
	}
}

DWORD CIPhysMemory::Attach( LPVOID p )
{
	if( ! m_p )
	{
		m_bCreateOwner = FALSE;				// owner
		m_p	= p;							// mapped pointer
		m_status = E_OK;					// last status
		return m_status;
	}
	else{
		m_status = E_CONTEXT;
		return m_status;
	}
}

DWORD CIPhysMemory::Allocate( DWORD dwSize )
{
	if( ! m_p )
	{
		dwSize	= (dwSize+0xfff)&0xfffff000;	// clip 4KB
		m_bCreateOwner = TRUE;					// owner
		m_dwLen		= dwSize;
		m_p	= AllocateRtMemory( dwSize );
		if( !m_p )
		{
			m_status	= GetLastRtError();
			return m_status;
		}
		m_dwPhysicalAddress	= GetRtPhysicalAddress( m_p ,dwSize ,0 );
		m_status = E_OK;					// last status
		return m_status;
	}
	else{
		m_status = E_CONTEXT;
		return m_status;
	}
}

///////////////////////////////////////////////////////////////////////
// assignment

CIPhysMemory & CIPhysMemory::
					operator = (
	CIPhysMemory &			src)
{
	if (this != &src)	// beware of self-assignment
	{
		// copy class specific members
		m_p			= src.m_p;
		m_dwLen		= src.m_dwLen;
		m_dwPhysicalAddress	= src.m_dwPhysicalAddress;
		m_bCreateOwner = FALSE;
		m_status = src.m_status;
	}
	return *this;
}


///////////////////////////////////////////////////////////////////////
// GetPointer
// returns: pointer or NULL

LPVOID CIPhysMemory::GetPointer(void)
{
	if (m_p == 0)
	{
		m_status	= E_EXIST;
		return NULL;
	}
	
	m_status		= E_OK;
	return m_p;
}


///////////////////////////////////////////////////////////////////////
// GetSize
// returns: size in bytes or 0

DWORD CIPhysMemory::	GetSize(void)
{
	m_status		= E_OK;
	return m_dwLen;
}

DWORD  CIPhysMemory::	GetPhysicalAddress(void)
{
	m_status		= E_OK;
	return m_dwPhysicalAddress;
}



///////////////////////////////////////////////////////////////////////
// Delete (protected)

void CIPhysMemory::		Delete(void)
{
	if( m_bCreateOwner )
	{
		if( m_p )
		{
			FreeRtMemory( (LPVOID)m_p );
			m_p	= 0;
		}
	}

	m_status		= E_OK;
}

DWORD CIPhysMemory::	GetLastError(void) const
{
	return m_status;
}

BYTE CIPhysMemory::ReadByte( WORD wOffset )
{
	return *(volatile BYTE*)((BYTE*)GetPointer() + wOffset);
}
WORD CIPhysMemory::ReadWord( WORD wOffset )
{
	return *(volatile WORD*)((BYTE*)GetPointer() + wOffset);
}
DWORD CIPhysMemory::ReadDword( WORD wOffset )
{
	return *(volatile DWORD*)((BYTE*)GetPointer() + wOffset);
}
void CIPhysMemory::WriteByte( WORD wOffset ,BYTE byVal )
{
	*(volatile BYTE*)((BYTE*)GetPointer() + wOffset) = byVal;
}
void CIPhysMemory::WriteWord( WORD wOffset ,WORD wVal )
{
	*(volatile WORD*)((BYTE*)GetPointer() + wOffset) = wVal;
}
void CIPhysMemory::WriteDword( WORD wOffset ,DWORD dwVal )
{
	*(volatile DWORD*)((BYTE*)GetPointer() + wOffset) = dwVal;
}

