/*----------------------------------------------------------------

	INtime general purpose Shared interrupt thread BASE class


		Copyright(C) programed by Kazumix ,Micronet 2015

------------------------------------------------------------------*/
#include <rtpp.h>
#include <stdio.h>
#include "SharedIntThread.h"
#include "Exception.h"

CISharedIntThread::CISharedIntThread(void)
	: CIThread()
{
}

CISharedIntThread::~CISharedIntThread(void)
{
	if (InternalDetach())
	{
		Delete();
		m_hRTObject	= NULL_RTHANDLE;
	}
}

DWORD CISharedIntThread::Create(
	char *				pszName,
	WORD				wLevel,
	BYTE				byMaxInt,
	DWORD				dwMilliseconds,
	DWORD				dwStack)
{
	Detach();

	m_wLevel		= wLevel | SHARED_LEVEL;
	m_byMaxInt		= byMaxInt;
	m_dwMilliseconds= dwMilliseconds;

	return CIThread::Create(pszName, 0, dwStack);
}

void CISharedIntThread::Delete(void)
{
	RTHANDLE		hTemp;

	hTemp			= m_hRTObject;
	m_hRTObject		= NULL_RTHANDLE;
	if (hTemp != NULL_RTHANDLE)
		if (::ResetRtInterruptHandler(m_wLevel))
			m_status= E_OK;
		else {
			m_status= ::GetLastRtError();
			DeleteRtThread(NULL_RTHANDLE);
		}
	else
		m_status	= E_PARAM;

	CIObject::Delete();
}

BOOL CISharedIntThread::InitInstance(void)
{
	SetRtProcessMaxPriority(NULL_RTHANDLE, 0);

	// handle hardware exception
	HardwareException::Hook();

	// shared interrupt
	m_wLevel	= ::SetRtInterruptHandlerEx(m_wLevel, m_byMaxInt,
					(LPPROC)SharedInterruptHandler, this);
	if (BAD_LEVEL == m_wLevel)
	{
		Fail("Cannot set interrupt");
		m_status= ::GetLastRtError();
		return FALSE;
	}

	m_status		= E_OK;
	return TRUE;
}

BOOL CISharedIntThread::DoForever(void)
{
	try{
		if (!::WaitForRtInterrupt(m_wLevel, m_dwMilliseconds))
		{
			m_status	= ::GetLastRtError();
			if (m_status == E_TIME)
				return OnSignal(FALSE);
			else
				return FALSE;
		}
		else
			return OnSignal(TRUE);
	}
	catch( HardwareException e )
	{
		printf("hardware exception\n");
		e.dump();
		return TRUE;
	}
	catch( ... )
	{
		printf("over load ");
		return TRUE;
	}

}

BOOL CISharedIntThread::OnInterrupt(void)
{
	return TRUE;
}

BOOL CISharedIntThread::OnSignal( BOOL bSignal )
{
	return TRUE;
}

DWORD CISharedIntThread::Enable( BOOL bState )
{
	BOOL			bResult;

	if (bState)
		bResult		= ::EnableRtInterrupt(m_wLevel);
	else
		bResult		= ::DisableRtInterrupt(m_wLevel);
	if (bResult)
		m_status	= ::GetLastRtError();
	else
		m_status	= E_OK;
	return m_status;
}

__INTERRUPT void CISharedIntThread::SharedInterruptHandler(
	WORD			wCSRA,
	WORD			wLevel,
	LPVOID			pv)
{
	CISharedIntThread	*pIntThread;

	__SHARED_INTERRUPT_PROLOG();

	pIntThread = (CISharedIntThread*)pv;
	if( pIntThread->OnInterrupt() )
	{		
		SignalRtInterruptThread( wLevel );
	}

	__SHARED_INTERRUPT_RETURN();
}

